#include "pml_iop.h"
#include "pml_iop_config.h"
#include "pml_iop_tables.h"
#include "pml_iop_calculate.h"

int main(int argc, char *argv[])
{
   int i,j; 
   int configflag = 0; /* Config flag - if not raised set to default pml.cfg */
   int sun_theta_flag = 0, sen_theta_flag = 0, dphi_flag = 0, Rrs_flag = 0; 
   int result = 0; /* flag for successful iterations */
   float sun_theta, sen_theta, dphi; /* Geometry in radians */
   double Rrs[NB], rho_w[NB];
   double a[NB],bbp[NB],ady[NB],ap[NB];
   
   char configfname[200];
   
   for(i=0;i<argc;i++){
      if(strncmp(argv[i],"-help",5) == 0){
         printf("Usage: pml_iop_model [-help] [-config <config_file>]\n");
         printf("[-sun_theta <sun_theta_radians>] [-sen_theta <sen_theta_radians>]\n");
         printf("[-dphi <dphi_radians>] [-Rrs <rrs412 rrs443 rrs490 rrs510 rrs555 rrs670>]\n");
         printf("[-rhow <rhow412 rhow443 rhow490 rhow510 rhow555 rhow670>]\n");
         
         exit(0);
      }
      if(strncmp(argv[i],"-config",7) == 0){
         sprintf(configfname,"%s",argv[i+1]);
         configflag = 1;
      } 
      if(strncmp(argv[i],"-sun_theta",8) == 0){
         sscanf(argv[i+1],"%f",&sun_theta);
         sun_theta_flag = 1;
      }
      if(strncmp(argv[i],"-sen_theta",8) == 0){
         sscanf(argv[i+1],"%f",&sen_theta);
         sen_theta_flag = 1;
      }
      if(strncmp(argv[i],"-dphi",8) == 0){
         sscanf(argv[i+1],"%f",&dphi);
         dphi_flag = 1;
      }
      if(strncmp(argv[i],"-Rrs",4) == 0){
         for(j=0;j<NB;j++){     
            sscanf(argv[i+(j+1)], "%lf", &Rrs[j]);
         }
         Rrs_flag = 1;
      }
      if(strncmp(argv[i],"-rhow",5) == 0){
         for(j=0;j<NB;j++){     
            sscanf(argv[i+(j+1)], "%lf", &rho_w[j]);
         }
      }   
   }

   if(configflag == 0){
       sprintf(configfname,"%s",DEFAULT_CFG);
   }

   for(i=0;i<NB;i++){
      if(Rrs_flag) rho_w[i] = M_PI*Rrs[i];
   }

   /* If the solar, satellite and azimuth angles not passed in set to default */
   if(sun_theta_flag == 0) sun_theta = 0.785; /* Solar zenith of 45 degrees */
   if(sen_theta_flag == 0) sen_theta = 0.0;
   if(dphi_flag == 0) dphi = 0.0; 
   
   /* Load the various LUTs into memory */
   load_work_tab(configfname);
   
   /* Load the various parameters into memory */
   load_config(configfname);
   
   /* call iop_model - the matrices a, bbp, ady and ap hold spectral info*/
   result = iop_model(rho_w,sun_theta,sen_theta,dphi,a,bbp,ady,ap);
   
   /* printf("a412 a443 a490 a510 a555 a670 ady412 ady443 ady490 ady510 ady555 ady670 ap412 ap443 ap490 ap510 ap555 ap670 bbp412 bbp443 bbp490 bbp510 bbp555 bbp670\n"); */
   if (result == 0){
      for(i=0;i<NB;i++){
         printf("%f ", a[i]+a_w[i]);
      }
      for(i=0;i<NB;i++){
         printf("%f ", ady[i]);
      }
      for(i=0;i<NB;i++){
         printf("%f ", ap[i]);
      }
      for(i=0;i<NB;i++){
         printf("%f ", bbp[i]);
      }
   }

   printf("\n");
}

